// -*- C++ -*-
#include <iostream>
#include "Rivet/Analysis.hh"
#include "Rivet/Projections/Beam.hh"
#include "Rivet/Projections/UnstableFinalState.hh"
#include "Rivet/ParticleName.hh"

namespace Rivet {


  /// @brief Production of the $\eta'(958)$ and $f_0(980)$ in $e^+e^-$ annihilation in the Upsilon region
  /// @author Peter Richardson
  class ARGUS_1993_S2669951 : public Analysis {
  public:

    ARGUS_1993_S2669951()
      : Analysis("ARGUS_1993_S2669951")
    {   }


    void init() {
      addProjection(Beam(), "Beams");
      addProjection(UnstableFinalState(), "UFS");

        _count_etaPrime_highZ = vector<CounterPtr>(2, bookCounter("TMP/counter"));
        _count_etaPrime_allZ = vector<CounterPtr>(3, bookCounter("TMP/counter"));
        _count_f0 = vector<CounterPtr>(3, bookCounter("TMP/counter"));
        _weightSum_cont = bookCounter("TMP/counter");
        _weightSum_Ups1 = bookCounter("TMP/counter");
        _weightSum_Ups2 = bookCounter("TMP/counter");

      _hist_cont_f0 = bookHisto1D(2, 1, 1);
      _hist_Ups1_f0 = bookHisto1D(3, 1, 1);
      _hist_Ups2_f0 = bookHisto1D(4, 1, 1);
    }


    void analyze(const Event& e) {

      const Beam beamproj = applyProjection<Beam>(e, "Beams");
      const double s = sqr(beamproj.sqrtS());
      const double roots = sqrt(s);
      const UnstableFinalState& ufs = applyProjection<UnstableFinalState>(e, "UFS");

      // Find the upsilons
      Particles upsilons;
      // First in unstable final state
      foreach (const Particle& p, ufs.particles())
        if (p.pid() == 553 || p.pid() == 100553 ) upsilons.push_back(p);
      // Then in whole event if fails
      if (upsilons.empty()) {
        foreach (const GenParticle* p, Rivet::particles(e.genEvent())) {
          if ( p->pdg_id() != 553 && p->pdg_id() != 100553 ) continue;
          const GenVertex* pv = p->production_vertex();
          bool passed = true;
          if (pv) {
            foreach (const GenParticle* pp, particles_in(pv)) {
              if ( p->pdg_id() == pp->pdg_id() ) {
                passed = false;
                break;
              }
            }
          }
          if (passed) upsilons.push_back(Particle(*p));
        }
      }
      // Continuum
      if (upsilons.empty()) {
        _weightSum_cont->fill();
        unsigned int nEtaA(0),nEtaB(0),nf0(0);
        foreach (const Particle& p, ufs.particles()) {
          int id = p.abspid();
          double xp = 2.*p.E()/roots;
          double beta = p.p3().mod() / p.E();
          if (id == 9010221) {
            _hist_cont_f0->fill(xp, 1.0/beta);
            ++nf0;
          } else if (id == 331) {
            if (xp > 0.35) ++nEtaA;
            ++nEtaB;
          }
        }
        _count_f0[2]             ->fill(nf0);
        _count_etaPrime_highZ[1] ->fill(nEtaA);
        _count_etaPrime_allZ[2]  ->fill(nEtaB);
      }
      else {
        // Find an Upsilon
        foreach (const Particle& ups, upsilons) {
          int parentId = ups.pid();
          ((parentId == 553) ? _weightSum_Ups1 : _weightSum_Ups2)->fill();
          Particles unstable;
          // Find the decay products we want
          findDecayProducts(ups.genParticle(), unstable);
          LorentzTransform cms_boost;
          if (ups.p3().mod() > 1*MeV)
            cms_boost = LorentzTransform(-ups.momentum().boostVector());
          double mass = ups.mass();
          unsigned int nEtaA(0), nEtaB(0), nf0(0);
          foreach(const Particle& p , unstable) {
            const int id = p.abspid();
            FourMomentum p2 = cms_boost.transform(p.momentum());
            double xp = 2.*p2.E()/mass;
            double beta = p2.p3().mod()/p2.E();
            if (id == 9010221) {
              ((parentId == 553) ? _hist_Ups1_f0 : _hist_Ups2_f0)->fill(xp, 1.0/beta);
              ++nf0;
            } else if (id == 331) {
              if (xp > 0.35) ++nEtaA;
              ++nEtaB;
            }
          }
          if (parentId == 553) {
            _count_f0[0]             ->fill(  nf0);
            _count_etaPrime_highZ[0] ->fill(nEtaA);
            _count_etaPrime_allZ[0]  ->fill(nEtaB);
          } else {
            _count_f0[1]->fill(nf0);
            _count_etaPrime_allZ[1]  ->fill(nEtaB);
          }
        }
      }
    }


    void finalize() {

      /// @todo Better to group these by coherent 'if (weightSum_X)' statements?

      // High-Z eta' multiplicity
      Scatter2DPtr s111 = bookScatter2D(1, 1, 1, true);
      if (_weightSum_Ups1->val() > 0) // Point at 9.460
        s111->point(0).setY(_count_etaPrime_highZ[0]->val() / _weightSum_Ups1->val(), 0);
      if (_weightSum_cont->val() > 0) // Point at 9.905
        s111->point(1).setY(_count_etaPrime_highZ[1]->val() / _weightSum_cont->val(), 0);


      // All-Z eta' multiplicity
      Scatter2DPtr s112 = bookScatter2D(1, 1, 2, true);
      if (_weightSum_Ups1->val() > 0) // Point at 9.460
        s112->point(0).setY(_count_etaPrime_allZ[0]->val() / _weightSum_Ups1->val(), 0);
      if (_weightSum_cont->val() > 0) // Point at 9.905
        s112->point(1).setY(_count_etaPrime_allZ[2]->val() / _weightSum_cont->val(), 0);
      if (_weightSum_Ups2->val() > 0) // Point at 10.02
        s112->point(2).setY(_count_etaPrime_allZ[1]->val() / _weightSum_Ups2->val(), 0);

      // f0 multiplicity
      Scatter2DPtr s511 = bookScatter2D(5, 1, 1, true);
      if (_weightSum_Ups1->val() > 0) // Point at 9.46
        s112->point(0).setY(_count_f0[0]->val() / _weightSum_Ups1->val(), 0);
      if (_weightSum_Ups2->val() > 0) // Point at 10.02
        s112->point(1).setY(_count_f0[1]->val() / _weightSum_Ups2->val(), 0);
      if (_weightSum_cont->val() > 0) // Point at 10.45
        s112->point(2).setY(_count_f0[2]->val() / _weightSum_cont->val(), 0);

      if (_weightSum_cont->val() > 0.) scale(_hist_cont_f0, 1./_weightSum_cont->val());
      if (_weightSum_Ups1->val() > 0.) scale(_hist_Ups1_f0, 1./_weightSum_Ups1->val());
      if (_weightSum_Ups2->val() > 0.) scale(_hist_Ups2_f0, 1./_weightSum_Ups2->val());
    }


  private:

    /// @name Counters
    //@{
    vector<CounterPtr> _count_etaPrime_highZ, _count_etaPrime_allZ, _count_f0;
    CounterPtr _weightSum_cont,_weightSum_Ups1,_weightSum_Ups2;
    //@}


    /// Histos
    Histo1DPtr _hist_cont_f0, _hist_Ups1_f0, _hist_Ups2_f0;


    /// Recursively walk the HepMC tree to find decay products of @a p
    void findDecayProducts(const GenParticle* p, Particles& unstable) {
      const GenVertex* dv = p->end_vertex();
      /// @todo Use better looping
      for (GenVertex::particles_out_const_iterator pp = dv->particles_out_const_begin(); pp != dv->particles_out_const_end(); ++pp) {
        const int id = abs((*pp)->pdg_id());
        if (id == 331 || id == 9010221) unstable.push_back(Particle(*pp));
        else if ((*pp)->end_vertex()) findDecayProducts(*pp, unstable);
      }
    }


  };


  // The hook for the plugin system
  DECLARE_RIVET_PLUGIN(ARGUS_1993_S2669951);

}
