// -*- C++ -*-
#include "Rivet/Analysis.hh"
#include "Rivet/Projections/ZFinder.hh"

namespace Rivet {


  /// @brief CMS Z pT and rapidity in Drell-Yan events at 7 TeV
  /// @author Justin Hugon, Luca Perrozzi
  class CMS_2012_I941555 : public Analysis {
  public:

    /// Constructor
    CMS_2012_I941555()
      : Analysis("CMS_2012_I941555")
    {}


    /// @name Analysis methods
    //@{

    void init() {

      // Set up projections
      /// @todo Really?: ZFinder zfinder_dressed_mu_pt(-2.1, 2.1, 20, PID::MUON, 60*GeV, 120*GeV, 0.2, false, true);
      FinalState fs;
      Cut cuts = Cuts::abseta < 2.1 && Cuts::pT > 20*GeV;
      ZFinder zfinder_dressed_mu_pt(fs, cuts, PID::MUON, 60*GeV, 120*GeV, 0.2);
      addProjection(zfinder_dressed_mu_pt, "ZFinder_dressed_mu_pt");
      ZFinder zfinder_dressed_el_pt(fs, cuts, PID::ELECTRON, 60*GeV, 120*GeV, 0.1);
      addProjection(zfinder_dressed_el_pt, "ZFinder_dressed_el_pt");

      ZFinder zfinder_dressed_mu_rap(fs, Cuts::open(), PID::MUON, 60*GeV, 120*GeV, 0.1);
      addProjection(zfinder_dressed_mu_rap, "ZFinder_dressed_mu_rap");
      ZFinder zfinder_dressed_el_rap(fs, Cuts::open(), PID::ELECTRON, 60*GeV, 120*GeV, 0.1);
      addProjection(zfinder_dressed_el_rap, "ZFinder_dressed_el_rap");

      _sumw_mu_dressed_pt  = bookCounter("TMP/sumw_mu_dressed_pt");
      _sumwpeak_mu_dressed = bookCounter("TMP/sumwpeak_mu_dressed");
      _sumw_el_dressed_rap = bookCounter("TMP/sumw_el_dressed_rap");
      _sumw_el_dressed_pt  = bookCounter("TMP/sumw_el_dressed_pt");
      _sumwpeak_el_dressed = bookCounter("TMP/sumwpeak_el_dressed");

      // Book histograms
      _hist_zrap_mu_dressed      = bookHisto1D(1, 1, 1);  // muon "dressed" rapidity
      _hist_zrap_el_dressed      = bookHisto1D(1, 1, 2);  // electron "dressed" rapidity
      _hist_zrap_comb_dressed    = bookHisto1D(1, 1, 3);  // electron "dressed" rapidity

      _hist_zpt_mu_dressed       = bookHisto1D(2, 1, 1);  // muon "dressed" pt
      _hist_zpt_el_dressed       = bookHisto1D(2, 1, 2);  // electron "dressed" pt
      _hist_zpt_comb_dressed     = bookHisto1D(2, 1, 3);  // electron "dressed" pt

      _hist_zptpeak_mu_dressed   = bookHisto1D(3, 1, 1);  // muon "dressed" pt peak
      _hist_zptpeak_el_dressed   = bookHisto1D(3, 1, 2);  // electron "dressed" pt peak
      _hist_zptpeak_comb_dressed = bookHisto1D(3, 1, 3);  // electron "dressed" pt peak
    }


    /// Do the analysis
    void analyze(const Event& evt) {

      const ZFinder& zfinder_dressed_mu_rap = applyProjection<ZFinder>(evt, "ZFinder_dressed_mu_rap");
      if (!zfinder_dressed_mu_rap.bosons().empty()) {
        _sumw_mu_dressed_rap->fill();
        const FourMomentum pZ = zfinder_dressed_mu_rap.bosons()[0].momentum();
        _hist_zrap_mu_dressed->fill(pZ.rapidity()/GeV);
        _hist_zrap_comb_dressed->fill(pZ.rapidity()/GeV);
      }

      const ZFinder& zfinder_dressed_mu_pt = applyProjection<ZFinder>(evt, "ZFinder_dressed_mu_pt");
      if (!zfinder_dressed_mu_pt.bosons().empty()) {
        _sumw_mu_dressed_pt->fill();
        const FourMomentum pZ = zfinder_dressed_mu_pt.bosons()[0].momentum();
        _hist_zpt_mu_dressed->fill(pZ.pT()/GeV);
        _hist_zpt_comb_dressed->fill(pZ.pT()/GeV);
        if (pZ.pT() < 30*GeV) {
          _sumwpeak_mu_dressed->fill();
          _hist_zptpeak_mu_dressed->fill(pZ.pT()/GeV);
          _hist_zptpeak_comb_dressed->fill(pZ.pT()/GeV);
        }
      }

      const ZFinder& zfinder_dressed_el_rap = applyProjection<ZFinder>(evt, "ZFinder_dressed_el_rap");
      if (!zfinder_dressed_el_rap.bosons().empty()) {
        _sumw_el_dressed_rap->fill();
        const FourMomentum pZ = zfinder_dressed_el_rap.bosons()[0].momentum();
        _hist_zrap_el_dressed->fill(pZ.rapidity()/GeV);
        _hist_zrap_comb_dressed->fill(pZ.rapidity()/GeV);
      }

      const ZFinder& zfinder_dressed_el_pt = applyProjection<ZFinder>(evt, "ZFinder_dressed_el_pt");
      if (!zfinder_dressed_el_pt.bosons().empty()) {
        _sumw_el_dressed_pt->fill();
        const FourMomentum pZ = zfinder_dressed_el_pt.bosons()[0].momentum();
        _hist_zpt_el_dressed->fill(pZ.pT()/GeV);
        _hist_zpt_comb_dressed->fill(pZ.pT()/GeV);
        if (pZ.pT() < 30*GeV) {
          _sumwpeak_el_dressed->fill();
          _hist_zptpeak_el_dressed->fill(pZ.pT()/GeV);
          _hist_zptpeak_comb_dressed->fill(pZ.pT()/GeV);
        }
      }

    }


    void finalize() {
      scale(_hist_zrap_mu_dressed, safediv(1, _sumw_mu_dressed_rap->val(), 1));
      scale(_hist_zpt_mu_dressed, safediv(1, _sumw_mu_dressed_pt->val(), 1));
      scale(_hist_zptpeak_mu_dressed, safediv(1, _sumwpeak_mu_dressed->val(), 1));

      scale(_hist_zrap_el_dressed, safediv(1, _sumw_el_dressed_rap->val(), 1));
      scale(_hist_zpt_el_dressed, safediv(1, _sumw_el_dressed_pt->val(), 1));
      scale(_hist_zptpeak_el_dressed, safediv(1, _sumwpeak_el_dressed->val(), 1));

      scale(_hist_zrap_comb_dressed, safediv(1, _sumw_el_dressed_rap->val()+_sumw_mu_dressed_rap->val(), 1));
      scale(_hist_zpt_comb_dressed, safediv(1, _sumw_el_dressed_pt->val()+_sumw_mu_dressed_pt->val(), 1));
      scale(_hist_zptpeak_comb_dressed, safediv(1, _sumwpeak_el_dressed->val()+_sumwpeak_mu_dressed->val(), 1));
    }

    //@}


  private:

    CounterPtr _sumw_mu_dressed_rap;
    CounterPtr _sumw_mu_dressed_pt;
    CounterPtr _sumwpeak_mu_dressed;

    CounterPtr _sumw_el_dressed_rap;
    CounterPtr _sumw_el_dressed_pt;
    CounterPtr _sumwpeak_el_dressed;

    Histo1DPtr _hist_zrap_mu_dressed;
    Histo1DPtr _hist_zpt_mu_dressed;
    Histo1DPtr _hist_zptpeak_mu_dressed;

    Histo1DPtr _hist_zrap_el_dressed;
    Histo1DPtr _hist_zpt_el_dressed;
    Histo1DPtr _hist_zptpeak_el_dressed;

    Histo1DPtr _hist_zrap_comb_dressed;
    Histo1DPtr _hist_zpt_comb_dressed;
    Histo1DPtr _hist_zptpeak_comb_dressed;

  };


  DECLARE_RIVET_PLUGIN(CMS_2012_I941555);

}
